from .db import DorisConnector
from .tools import run_select_query, preview_table, describe_table,list_all_tables

__all__ = [DorisConnector, run_select_query, preview_table, describe_table, list_all_tables]


import os
import sys
import asyncio
from doris_mcp_lite import config
from doris_mcp_lite.mcp_app import mcp
from doris_mcp_lite import config
from doris_mcp_lite.db import tools, DorisConnector
from doris_mcp_lite.res import resources
from doris_mcp_lite.prompts import general_prompts, customize_prompts
import traceback

class MCPDorisServer:

    def __init__(self):
        self.server = mcp


    async def _test_db_connection(self):
        """
        测试数据库连接是否成功。
        """
        try:
            async with DorisConnector() as conn:
                result = await conn.execute_query("SELECT 1")
                if result:
                    print("✅ Database connection successful.")
                else:
                    raise Exception("Database connection test failed: please config .env file.")
        except Exception as e:
            print("❌ Database connection test failed.")
            raise e


    async def run_async(self):
        """
        启动 MCP Server
        """
        try:
            print("🚀 Doris MCP Server is starting...")
            await self._test_db_connection()
            await self.server.run_stdio_async()
        except Exception as e:
            print("🚨 Doris MCP Server failed to start.")
            print(f"Error: {e}")
            traceback.print_exc()


async def main():
    if len(sys.argv) > 1:
        os.environ["DORIS_URL"] = sys.argv[1]

    config.init_config()

    server = MCPDorisServer()
    await server.run_async()


from .config import ensure_env_file, load_environment, get_db_config
import os

DB_CONFIG = None
MCP_SERVER_NAME = None
DEBUG = None

def init_config():
    global DB_CONFIG, MCP_SERVER_NAME, DEBUG
    ensure_env_file()
    load_environment()
    DB_CONFIG = get_db_config()
    MCP_SERVER_NAME = os.getenv("MCP_SERVER_NAME", "DorisAnalytics")
    DEBUG = os.getenv("DEBUG", "false").lower() in ["1", "true", "yes"]

__all__ = ["DB_CONFIG", "MCP_SERVER_NAME", "DEBUG"]


from doris_mcp_lite.mcp_app import mcp
from mcp.server.fastmcp.prompts import base



# 1. 简单表格查询模板
@mcp.prompt()
def query_table_overview(table_name: str) -> str:
    """
    查看表格的结构和前几行数据
    """
    return (
        f"请查询 `{table_name}` 表的结构信息，并展示前 10 行数据。"
        f"\n确保格式清晰，使用表格展示内容。"
    )

@mcp.prompt(name = "table_comment_overview", description = "查看表的注释信息")
def query_table_comment(table_name: str) -> str:
    """
    查看表格的注释信息
    """
    return f"请查询 `{table_name}` 表的注释信息，这些信息用于描述表的用途。"

# 2. 汇总统计模板
@mcp.prompt()
def summarize_column(column: str, table: str) -> str:
    """
    统计某列的常见汇总信息
    """
    return f"请统计 `{table}` 表中 `{column}` 列的最大值、最小值、平均值、标准差与总和。"


# 3. 分组聚合模板
@mcp.prompt()
def groupby_analysis(group_col: str, target_col: str, table: str) -> str:
    """
    对某字段进行分组聚合分析
    """
    return (
        f"请对 `{table}` 表按 `{group_col}` 分组，统计每组在 `{target_col}` 上的均值与总和，"
        f"并按总和降序排序。"
    )


# 4. 数据趋势分析模板（近7日、近30日）
@mcp.prompt()
def trend_analysis(date_col: str, metric_col: str, table: str, days: int = 7) -> str:
    """
    绘制最近几天某指标的趋势
    """
    return (
        f"请分析 `{table}` 表中最近 {days} 天的 `{metric_col}` 变化趋势，"
        f"日期字段为 `{date_col}`。可视化建议使用折线图。"
    )


# 5. 多轮对话模板：异常值分析
@mcp.prompt()
def outlier_analysis() -> list[base.Message]:
    return [
        base.UserMessage("我想分析某个表中的异常值"),
        base.AssistantMessage("好的，请告诉我要分析哪个表，以及哪一列？"),
        base.UserMessage("表是 user_metrics，列是 daily_usage"),
        base.AssistantMessage(
            "请稍等，我将检测 `user_metrics` 表中 `daily_usage` 列的异常值，"
            "可能使用箱线图或 3σ 原则。"
        ),
    ]


# 6. 多轮对话模板：多表联合查询
@mcp.prompt()
def multi_table_join_query() -> list[base.Message]:
    return [
        base.UserMessage("我想做一个多表联合查询"),
        base.AssistantMessage("好的，请告诉我涉及哪些表，以及它们之间的关联字段？"),
        base.UserMessage("比如 user_info 和 user_logs，关联字段是 user_id"),
        base.AssistantMessage(
            "明白了，我会生成一个 SQL 来将 `user_info` 表与 `user_logs` 表通过 `user_id` 字段进行关联，"
            "并可选地限制字段或加上筛选条件。是否有特定字段或条件要查询？"
        ),
    ]



import asyncio
import argparse
import json
import urllib.parse
from mcp import ClientSession
from mcp.client.stdio import stdio_client
from mcp import StdioServerParameters
from doris_mcp_lite.db import DorisConnector
from doris_mcp_lite.config import *


async def test_resources(session: ClientSession):
    print("== 列出所有资源 ==")
    resources = await session.list_resources()
    for res in resources.resources:
        print(f"- {res.uri} ({res.name})")

    if resources.resources:
        print("\n== 读取第一个资源 ==")
        uri = resources.resources[0].uri
        content, mime_type = await session.read_resource(uri)
        print(f"内容类型: {mime_type}\n内容预览:\n{content[:500]}...")



async def test_tools(session: ClientSession):
    print("== 列出所有工具 ==")
    tools = await session.list_tools()
    for tool in tools.tools:
        print(f"- {tool.name}: {tool.description}")

    if tools.tools:
        tool = tools.tools[0]
        print(f"\n== 调用第一个工具：{tool.name} ==")

        # 构造默认的空参数，如果需要可以在这里根据schema生成
        args = {}
        for key, schema in tool.inputSchema.get("properties", {}).items():
            if schema.get("type") == "string":
                args[key] = "test"
            elif schema.get("type") == "integer":
                args[key] = 1
            elif schema.get("type") == "number":
                args[key] = 1.0
            elif schema.get("type") == "boolean":
                args[key] = True
            else:
                args[key] = None

        print(f"调用参数: {json.dumps(args)}")
        result = await session.call_tool(tool.name, arguments=args)
        print(f"返回结果: {result}")



async def test_prompts(session: ClientSession):
    print("== 列出所有提示词 (prompts) ==")
    prompts = await session.list_prompts()
    for prompt in prompts.prompts:
        print(f"- {prompt.name}: {prompt.description}")

    if prompts.prompts:
        prompt = prompts.prompts[0]
        print(f"\n== 获取第一个提示词：{prompt.name} ==")
        response = await session.get_prompt(prompt.name, arguments={})
        print("提示词返回内容:")
        print(response)




async def main():
    parser = argparse.ArgumentParser(description="MCP Server 测试工具")
    parser.add_argument("--server", type=str, required=True, help="Server 启动脚本路径，例如 server.py")
    parser.add_argument("--test", type=str, required=True, choices=["resources", "tools", "prompts", "all", "dbconfig"], help="要测试的功能")
    args = parser.parse_args()

    # 特殊测试：数据库配置解析
    if args.test == "dbconfig":
        test_db_config_from_uri(args.server)
        return

    # 连接到 MCP Server
    server_params = StdioServerParameters(command="python", args=[args.server])
    async with stdio_client(server_params) as (read_stream, write_stream):
        async with ClientSession(read_stream, write_stream) as session:
            await session.initialize()

            if args.test in ["resources", "all"]:
                await test_resources(session)
            if args.test in ["tools", "all"]:
                await test_tools(session)
            if args.test in ["prompts", "all"]:
                await test_prompts(session)


def test_db_config_from_uri(uri: str):
    """
    解析 URI 并打印对应的数据库配置，并尝试连接数据库
    """
    parsed = urllib.parse.urlparse(uri)
    config = {
        "host": parsed.hostname,
        "port": parsed.port,
        "user": parsed.username,
        "password": parsed.password,
        "database": parsed.path.lstrip("/")
    }

    print("== 数据库连接测试 ==")
    print("URI:", uri)
    for key, value in config.items():
        print(f"{key.capitalize()}: {value}")

    print("尝试连接数据库中...")
    try:
        with DorisConnector(config) as connector:
            result = asyncio.run(connector.execute_query("SELECT 1;"))
            print("✅ 数据库连接成功，返回结果:", result)
    except Exception as e:
        print("❌ 数据库连接失败:", e)

if __name__ == "__main__":
    asyncio.run(main())



import re
from mcp.server.fastmcp import Context
from doris_mcp_lite.db import DorisConnector
from doris_mcp_lite.mcp_app import mcp
from doris_mcp_lite.config import get_db_config




def _is_safe_select(sql: str) -> bool:
    """
    检查 SQL 是否为安全的 SELECT 查询
    """
    sql = sql.strip().lower()
    return sql.startswith("select") and not re.search(r"\b(update|delete|insert|drop|alter|create|replace|truncate)\b", sql)



@mcp.tool(name="run_select_query",description="run a read-only select query and return formatted result")
async def run_select_query(sql: str, ctx: Context) -> str:
    """
    执行只读 SELECT 查询并返回格式化结果。
    """
    if not _is_safe_select(sql):
        return "仅允许只读 SELECT 查询，不支持修改型语句。"

    try:
        async with DorisConnector() as db:
            rows = await db.execute_query(sql)
            if not rows:
                return "查询结果为空。"

            headers = rows[0].keys()
            lines = [" | ".join(headers)]
            lines.append("-" * len(lines[0]))

            total = len(rows)
            for idx, row in enumerate(rows):
                await ctx.report_progress(idx + 1, total)
                lines.append(" | ".join(str(row[col]) for col in headers))
            return "\n".join(lines)
    except Exception as e:
        return f"查询失败: {str(e)}"



@mcp.tool(name="preview_table",description="preview the first 10 rows of a table")
async def preview_table(table_name: str) -> str:
    """
    预览指定表前 10 行数据。
    """
    try:
        sql = f"SELECT * FROM {table_name} LIMIT 10;"
        return await run_select_query(sql)
    except Exception as e:
        return f"预览失败: {str(e)}"




@mcp.tool(name="describe_table",description="return the structure of a table, including field names, types, nullability, default values, and comments")
async def describe_table(table_name: str) -> str:
    """
    返回指定表的字段结构，包括字段名、类型、是否为 null、默认值和注释。
    """
    try:
        async with DorisConnector() as db:
            schema = await db.get_table_schema(table_name)
            if not schema:
                return f"表 `{table_name}` 不存在或无法获取结构信息。"

            headers = ["Field", "Type", "Null", "Key", "Default", "Extra"]
            lines = [" | ".join(headers)]
            lines.append("-" * len(lines[0]))

            for row in schema:
                line = " | ".join(str(row.get(h, "")) for h in headers)
                lines.append(line)

            return "\n".join(lines)

    except Exception as e:
        return f"获取表结构失败: {str(e)}"




@mcp.tool(name="list_all_tables",description="list all tables in the current database")
async def list_all_tables(db_name: str = None) -> str:
    """
    列出当前数据库的所有表。
    """
    if db_name is None:
        db_name = get_db_config()["database"]

    try:
        async with DorisConnector() as db:
            tables = await db.list_tables(db_name)
            return "\n".join(tables)
    except Exception as e:
        return f"无法获取表列表: {str(e)}"
    



import asyncio
import pymysql
from pymysql.cursors import DictCursor
from doris_mcp_lite.config import get_db_config
from dbutils.pooled_db import PooledDB
class DorisPool:
    _pool = None

    @classmethod
    def init_pool(cls, config):
        if cls._pool is None:
            cls._pool = PooledDB(
                creator=pymysql,
                maxconnections=8,
                mincached=2,
                blocking=True,
                host=config["host"],
                port=config["port"],
                user=config["user"],
                password=config["password"],
                database=config["database"],
                cursorclass=DictCursor,
                autocommit=True
            )

    @classmethod
    def get_connection(cls):
        if cls._pool is None:
            raise RuntimeError("Connection pool is not initialized")
        return cls._pool.connection()


class DorisConnector:
    def __init__(self, config: dict = None):
        self.config = config or get_db_config()
        DorisPool.init_pool(self.config)


    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass  # No cleanup needed for connection pool

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        pass

    async def execute_query(self, sql: str) -> list[dict]:
        def _run():
            with DorisPool.get_connection() as conn:
                with conn.cursor() as cursor:
                    cursor.execute(sql)
                    return cursor.fetchall()
        return await asyncio.to_thread(_run)

    async def get_table_schema(self, table_name: str) -> list[dict]:
        """
        获取指定表的字段信息，包括字段名、类型、是否为空、默认值等
        """
        sql = f"DESCRIBE {table_name};"
        return await self.execute_query(sql)

    async def list_tables(self, db: str) -> list[str]:
        """
        获取当前数据库中所有表的列表
        """
        sql = f"SHOW TABLES IN {db};"
        result = await self.execute_query(sql)
        return [row[f'Tables_in_{self.config["database"]}'] for row in result]

    async def list_columns(self, table_name: str) -> list[str]:
        """
        获取指定表的所有列名
        """
        sql = f"SHOW COLUMNS FROM {table_name};"
        result = await self.execute_query(sql)
        return [row['Field'] for row in result]


from doris_mcp_lite.mcp_app import mcp
from doris_mcp_lite.db import DorisConnector
from doris_mcp_lite.config import get_db_config
from typing import Optional

async def _get_table_schemas(db_name: str) -> dict[str, str]:
    """
    获取所有表的结构信息，返回一个字典：
    {
        "table_name": "字段名 | 类型 | 是否为空 ... \n ...",
        ...
    }
    """
    async with DorisConnector() as db:
        tables = await db.list_tables(db_name)
    result = {}

    for table in tables:
        try:
            async with DorisConnector() as db:
                schema = await db.get_table_schema(table)
            if not schema:
                continue

            headers = ["Field", "Type", "Null", "Key", "Default", "Extra"]
            lines = [" | ".join(headers)]
            lines.append("-" * len(lines[0]))

            for row in schema:
                line = " | ".join(str(row.get(h, "")) for h in headers)
                lines.append(line)

            result[table] = "\n".join(lines)

        except Exception as e:
            result[table] = f"无法获取表结构: {str(e)}"

    return result




async def _get_table_comments(db_name: str) -> dict[str, str]:
    """
    获取所有表的注释信息，返回一个字典：
    {
        "table_name": "表注释",
        ...
    }
    """
    sql = f"""
    SELECT TABLE_NAME, TABLE_COMMENT
    FROM information_schema.TABLES
    WHERE TABLE_SCHEMA = '{db_name}'
    """
    try:
        async with DorisConnector() as db:
            results = await db.execute_query(sql)
        return {row["TABLE_NAME"]: row["TABLE_COMMENT"] or "无注释" for row in results}
    except Exception as e:
        return {"error": f"无法获取表注释信息: {str(e)}"}
    



@mcp.resource("doris://schema/{db_name}")
async def all_table_schemas(db_name: str = None) -> str:
    """
    返回指定数据库下所有表的结构。
    """
    if db_name is None:
        db_name = get_db_config()["database"]

    schemas = await _get_table_schemas(db_name)

    content = []
    for table_name, schema_text in schemas.items():
        content.append(f"# 表: {table_name}\n{schema_text}\n")

    return "\n\n".join(content)




@mcp.resource("doris://schema/{table}")
async def table_schema(table: str) -> Optional[str]:
    """
    返回单个表的字段结构信息。
    """
    try:
        async with DorisConnector() as db:
            schema = await db.get_table_schema(table)
            if not schema:
                return f"表 `{table}` 不存在或无结构信息。"

            headers = ["Field", "Type", "Null", "Key", "Default", "Extra"]
            lines = [" | ".join(headers)]
            lines.append("-" * len(lines[0]))

            for row in schema:
                lines.append(" | ".join(str(row.get(h, "")) for h in headers))

            return f"# 表: {table}\n" + "\n".join(lines)

    except Exception as e:
        return f"无法获取表 `{table}` 的结构信息: {str(e)}"
    


@mcp.resource("doris://table-rowcount/{table}")
async def get_table_rowcount(table: str) -> str:
    """
    返回指定表的行数。
    """
    try:
        async with DorisConnector() as db:
            sql = f"SELECT COUNT(*) AS row_count FROM {table}"
            result = await db.execute_query(sql)
            if result:
                return f"表 `{table}` 的行数为: {result[0]['row_count']}"
            else:
                return f"表 `{table}` 不存在或无法获取行数。"
    except Exception as e:
        return f"获取表 `{table}` 行数失败: {str(e)}"


@mcp.resource("doris://table-comments/{db_name}")
async def all_table_comments(db_name: str = None) -> str:
    """
    返回指定数据库下所有表的注释信息。
    """
    if db_name is None:
        db_name = get_db_config()["database"]
    comments = await _get_table_comments(db_name)
    content = []
    for table_name, comment in comments.items():
        content.append(f"- {table_name}: {comment}")
    return "\n".join(content)


# config.py
import os
from dotenv import load_dotenv
from pathlib import Path
import shutil
import re

# 确定.env路径和.env.example路径
env_dir = Path(__file__).resolve().parent
env_path = env_dir / ".env"
env_example_path = env_dir / ".env.example"


def load_environment():
    """
    加载 .env 配置
    """
    load_dotenv(dotenv_path=env_path)



def ensure_env_file():
    """
    确保 .env 文件存在，如果不存在则从 .env.example 复制
    """
    if not env_path.exists():
        if env_example_path.exists():
            shutil.copy(env_example_path, env_path)
            print(f"未找到 .env 文件，已自动从 .env.example 创建 {env_path}")
            print(f"请根据实际情况编辑 {env_path}，填写正确的数据库连接信息。")
        else:
            raise FileNotFoundError(
                f"❌ 配置文件 {env_path} 不存在，且找不到示例文件 {env_example_path}。\n"
                f"请手动创建 .env 文件或联系开发者提供模板。"
            )



# 尝试解析 DORIS_URL（优先级高于单独的 DB_HOST 等）
def parse_doris_url(url: str):
    """
    解析形如 doris://user:pass@host:port/dbname 的连接字符串
    """
    pattern = r"^doris:\/\/(?P<user>[^:]+):(?P<password>[^@]+)@(?P<host>[^:]+):(?P<port>\d+)\/(?P<database>\w+)$"
    match = re.match(pattern, url)
    if not match:
        raise ValueError("❌ DORIS_URL 格式不正确，应为 doris://user:pass@host:port/dbname")
    return {
        "host": match.group("host"),
        "port": int(match.group("port")),
        "user": match.group("user"),
        "password": match.group("password"),
        "database": match.group("database"),
    }



def get_db_config():
    """
    获取数据库连接配置，优先解析 DORIS_URL
    """
    if os.getenv("DORIS_URL"):
        return parse_doris_url(os.getenv("DORIS_URL"))
    else:
        return {
            "host": os.getenv("DB_HOST", "localhost"),
            "port": int(os.getenv("DB_PORT", 9030)),
            "user": os.getenv("DB_USER", "root"),
            "password": os.getenv("DB_PASSWORD", ""),
            "database": os.getenv("DB_NAME", ""),
        }



from .general_prompts import query_table_overview, summarize_column, groupby_analysis, trend_analysis, outlier_analysis

__all__ = [query_table_overview, summarize_column, groupby_analysis, trend_analysis, outlier_analysis]